-- Model --

create table flows (
  _id           integer primary key autoincrement,
  version       integer not null default 1,
  title         text,
  description   text,
  logging       integer, -- version 4
  community_id  integer,
  downloaded    integer,
  statements    integer not null default 0, -- version 2
  data          blob not null
)
;;

create table fibers (
  _id           integer primary key autoincrement,
  flow_id       integer not null,
  flow_version  integer not null,
  statement_id  integer not null,
  started_at    integer not null,
  parent_id     integer, -- version 3
  return_to     integer, -- version 6
  data          blob not null
)
;;
-- version 2
create index fibers_flow_id_flow_version
    on fibers (flow_id, flow_version)
;;
-- version 3
create index fibers_parent_id
    on fibers (parent_id)
;;

create table cleanups (
  _id           integer primary key,
  flow_id       integer not null,
  flow_version  integer not null,
  flow_data     blob not null,
  deleted       integer -- version 4
)
;;

-- version 5
create table variables (
  _id           integer primary key,
  flow_id       integer not null,
  flow_version  integer not null,
  register      integer not null,
  data          blob not null
)
;;
create unique index variables_flow_id_flow_version_register
    on variables (flow_id, flow_version, register)
;;


-- Triggers --

create trigger before_update_flows_data
before update of data on flows
begin
  insert into cleanups ( flow_id, flow_version, flow_data )
  values ( old._id, old.version, old.data )
  ;
  update flows
     set version = old.version + 1
   where _id = old._id
  ;
end
;;

create trigger before_delete_flows
before delete on flows
begin
  insert into cleanups ( flow_id, flow_version, flow_data, deleted )
  values ( old._id, old.version, old.data, 1 )
  ;
end
;;

-- version 5
create trigger after_delete_cleanups
after delete on cleanups
begin
  delete from fibers
   where flow_id = old.flow_id
     and flow_version = old.flow_version
  ;
  delete from variables
   where flow_id = old.flow_id
     and flow_version = old.flow_version
  ;
end
;;

